@@ -91,7 +91,7 @@ def Slicer(volume,
     visibles = [None, None, None]
     msh = volume.zSlice(int(dims[2]/2))
     msh.alpha(alpha).lighting('', la, ld, 0)
-    msh.pointColors(cmap=_cmap_slicer, vmin=rmin, vmax=rmax)
+    msh.cmap(_cmap_slicer, vmin=rmin, vmax=rmax)
     if map2cells: msh.mapPointsToCells()
     vp.renderer.AddActor(msh)
     visibles[2] = msh
@@ -100,7 +100,7 @@ def Slicer(volume,
     def sliderfunc_x(widget, event):
         i = int(widget.GetRepresentation().GetValue())
         msh = volume.xSlice(i).alpha(alpha).lighting('', la, ld, 0)
-        msh.pointColors(cmap=_cmap_slicer, vmin=rmin, vmax=rmax)
+        msh.cmap(_cmap_slicer, vmin=rmin, vmax=rmax)
         if map2cells: msh.mapPointsToCells()
         vp.renderer.RemoveActor(visibles[0])
         if i and i<dims[0]: vp.renderer.AddActor(msh)
@@ -109,7 +109,7 @@ def Slicer(volume,
     def sliderfunc_y(widget, event):
         i = int(widget.GetRepresentation().GetValue())
         msh = volume.ySlice(i).alpha(alpha).lighting('', la, ld, 0)
-        msh.pointColors(cmap=_cmap_slicer, vmin=rmin, vmax=rmax)
+        msh.cmap(_cmap_slicer, vmin=rmin, vmax=rmax)
         if map2cells: msh.mapPointsToCells()
         vp.renderer.RemoveActor(visibles[1])
         if i and i<dims[1]: vp.renderer.AddActor(msh)
@@ -118,7 +118,7 @@ def Slicer(volume,
     def sliderfunc_z(widget, event):
         i = int(widget.GetRepresentation().GetValue())
         msh = volume.zSlice(i).alpha(alpha).lighting('', la, ld, 0)
-        msh.pointColors(cmap=_cmap_slicer, vmin=rmin, vmax=rmax)
+        msh.cmap(_cmap_slicer, vmin=rmin, vmax=rmax)
         if map2cells: msh.mapPointsToCells()
         vp.renderer.RemoveActor(visibles[2])
         if i and i<dims[2]: vp.renderer.AddActor(msh)
@@ -173,7 +173,7 @@ def Slicer(volume,
         _cmap_slicer = bu.status()
         for mesh in visibles:
             if mesh:
-                mesh.pointColors(cmap=_cmap_slicer, vmin=rmin, vmax=rmax)
+                mesh.cmap(_cmap_slicer, vmin=rmin, vmax=rmax)
                 if map2cells:
                     mesh.mapPointsToCells()
         vp.renderer.RemoveActor(mesh.scalarbar)
@@ -202,7 +202,7 @@ def Slicer(volume,
     comment = None
     if verbose:
         comment = Text2D("Use sliders to slice volume\nClick button to change colormap",
-                         font='Montserrat', s=0.8)
+                         font='', s=0.8)
 
     vp.show(msh, hist, comment, interactive=False)
     vp.interactive = True
@@ -447,7 +447,8 @@ def IsosurfaceBrowser(volume, c=None, alpha=1, lego=False, cmap='hot', pos=None)
         sliderpos = 4
         slidertitle = "threshold"
         showval = True
-        mesh = volume.isosurface(threshold=threshold).color(c).alpha(alpha)
+        mesh = volume.isosurface(threshold)
+        mesh.color(c).alpha(alpha)
 
     if pos is not None:
         sliderpos = pos
@@ -529,18 +530,17 @@ def Browser(meshes, sliderpos=((0.55, 0.07),(0.96, 0.07)), c=None):
     return vp
 
 
-
-
+########################################################################
 class Animation(Plotter):
     """
     Animate simultaneously various objects
     by specifying event times and durations of different visual effects.
-    
+
     See examples
     `here <https://github.com/marcomusy/vedo/blob/master/vedo/examples/other>`_.
-    
+
     |animation1| |animation2|
-    
+
     N.B.: this is still an experimental feature at the moment.
 
     A ``Plotter`` derived class that allows to animate simultaneously various objects
@@ -589,7 +589,7 @@ class Animation(Plotter):
             if self._lastActs:
                 objs = self._lastActs
             else:
-                printc('Need to specify actors!', c=1)
+                printc('Need to specify actors!', c='r')
                 raise RuntimeError
 
         objs2 = objs
@@ -786,7 +786,7 @@ class Animation(Plotter):
             elif style=='glossy'  : pars = [0.1, 0.7, 0.9, 90, c]
             elif style=='default' : pars = [0.1, 1.0, 0.05, 5, c]
             else:
-                printc('Unknown lighting style:', [style], c=1)
+                printc('Unknown lighting style:', [style], c='r')
 
             for tt in rng:
                 inputvalues = []
@@ -818,7 +818,7 @@ class Animation(Plotter):
         if self.bookingMode:
             acts, t, duration, rng = self._parse(act, t, duration)
             if len(acts) != 1:
-                printc('Error in move(), can move only one object.', c=1)
+                printc('Error in move(), can move only one object.', c='r')
             cpos = acts[0].pos()
             pt = np.array(pt)
             dv = (pt - cpos)/len(rng)
@@ -841,7 +841,7 @@ class Animation(Plotter):
         if self.bookingMode:
             acts, t, duration, rng = self._parse(act, t, duration)
             if len(acts) != 1:
-                printc('Error in rotate(), can move only one object.', c=1)
+                printc('Error in rotate(), can move only one object.', c='r')
             for tt in rng:
                 ang = angle/len(rng)
                 self.events.append((tt, self.rotate, acts, (axis, ang)))
@@ -876,7 +876,7 @@ class Animation(Plotter):
         if self.bookingMode:
             acts, t, duration, rng = self._parse(act, t, duration)
             if len(acts) != 1:
-                printc('Error in meshErode(), can erode only one object.', c=1)
+                printc('Error in meshErode(), can erode only one object.', c='r')
             diag = acts[0].diagonalSize()
             x0,x1, y0,y1, z0,z1 = acts[0].GetBounds()
             corners = [ (x0,y0,z0), (x1,y0,z0), (x1,y1,z0), (x0,y1,z0),
