@@ -1,6 +1,7 @@
 from __future__ import division, print_function
 import numpy as np
 import vtk
+import vedo
 import vedo.colors as colors
 import vedo.docs as docs
 import vedo.settings as settings
@@ -12,7 +13,15 @@ __doc__ = (
     + docs._defs
 )
 
-__all__ = ['Base3DProp','BaseActor','BaseGrid']
+__all__ = ['Base3DProp',
+           'BaseActor',
+           'BaseGrid',
+           "probePoints",
+           "probeLine",
+           "probePlane",
+           "streamLines",
+           ]
+
 
 ###############################################################################
 # classes
@@ -41,6 +50,7 @@ class Base3DProp(object):
         self._legend = None
         self.renderedAt = set()
         self.transform = None
+        self._set2actcam = False
 
 
     def pickable(self, value=None):
@@ -51,23 +61,6 @@ class Base3DProp(object):
             self.SetPickable(value)
             return self
 
-
-    def legend(self, txt=None):
-        """Set/get ``Mesh`` legend text.
-
-        :param str txt: legend text.
-
-        Size and positions can be modified by setting attributes
-        ``Plotter.legendSize``, ``Plotter.legendBC`` and ``Plotter.legendPos``.
-        """
-        if txt is None:
-            return self._legend
-        else:
-            self._legend = str(txt)
-            return self
-
-
-
     def time(self, t=None):
         """Set/get object's absolute time."""
         if t is None:
@@ -155,7 +148,6 @@ class Base3DProp(object):
             self._updateShadow()
         return self
 
-
     def rotate(self, angle, axis=(1, 0, 0), axis_point=(0, 0, 0), rad=False):
         """Rotate around an arbitrary `axis` passing through `axis_point`."""
         if rad:
@@ -189,7 +181,6 @@ class Base3DProp(object):
                            self.shadow.GetProperty().GetOpacity())
         return self
 
-
     def rotateX(self, angle, rad=False):
         """Rotate around x-axis. If angle is in radians set ``rad=True``."""
         if rad:
@@ -325,8 +316,8 @@ class Base3DProp(object):
             try:
                 self.SetUserTransform(T)
             except TypeError:
-                colors.printc('~times Error in setTransform():',
-                              'consider transformPolydata() instead.', c=1)
+                colors.printc('\times Error in setTransform():',
+                              'consider transformPolydata() instead.', c='r')
         return self
 
 
@@ -366,6 +357,12 @@ class Base3DProp(object):
         bx.wireframe()
         return bx
 
+    def useBounds(self, ub=True):
+        """Instruct the current camera to either take into account or ignore
+        the object bounds when resetting."""
+        self.SetUseBounds(ub)
+        return self
+
     def bounds(self):
         """Get the object bounds.
         Returns a list in format [xmin,xmax, ymin,ymax, zmin,zmax]."""
@@ -417,54 +414,60 @@ class Base3DProp(object):
         """Draw axes for the input object or for a specified range.
         Returns an ``Assembly`` object.
 
-        - `xtitle`,            ['x'], x-axis title text.
-        - `xrange`,           [None], x-axis range in format (xmin, ymin), default is automatic.
-        - `numberOfDivisions`,[None], approximate number of divisions on the longest axis
-        - `axesLineWidth`,       [1], width of the axes lines
-        - `gridLineWidth`,       [1], width of the grid lines
-        - `reorientShortTitle`, [True], titles shorter than 2 letter are placed horizontally
-        - `originMarkerSize`, [0.01], draw a small cube on the axis where the origin is
-        - `titleDepth`,          [0], extrusion fractional depth of title text
-        - `xyGrid`,           [True], show a gridded wall on plane xy
-        - `yzGrid`,           [True], show a gridded wall on plane yz
-        - `zxGrid`,           [True], show a gridded wall on plane zx
-        - `zxGrid2`,         [False], show zx plane on opposite side of the bounding box
-        - `xyGridTransparent`  [False], make grid plane completely transparent
-        - `xyGrid2Transparent` [False], make grid plane completely transparent on opposite side box
-        - `xyPlaneColor`,   ['gray'], color of the plane
-        - `xyGridColor`,    ['gray'], grid line color
-        - `xyAlpha`,          [0.15], grid plane opacity
-        - `showTicks`,        [True], show major ticks
-        - `xTitlePosition`,   [0.32], title fractional positions along axis
-        - `xTitleOffset`,     [0.05], title fractional offset distance from axis line
+        - `xtitle`,                ['x'], x-axis title text
+        - `xrange`,               [None], x-axis range in format (xmin, ymin), default is automatic.
+        - `numberOfDivisions`,    [None], approximate number of divisions on the longest axis
+        - `axesLineWidth`,           [1], width of the axes lines
+        - `gridLineWidth`,           [1], width of the grid lines
+        - `reorientShortTitle`,   [True], titles shorter than 2 letter are placed horizontally
+        - `titleDepth`,              [0], extrusion fractional depth of title text
+        - `xyGrid`,               [True], show a gridded wall on plane xy
+        - `yzGrid`,               [True], show a gridded wall on plane yz
+        - `zxGrid`,               [True], show a gridded wall on plane zx
+        - `zxGrid2`,             [False], show zx plane on opposite side of the bounding box
+        - `xyGridTransparent`    [False], make grid plane completely transparent
+        - `xyGrid2Transparent`   [False], make grid plane completely transparent on opposite side box
+        - `xyPlaneColor`,       ['gray'], color of the plane
+        - `xyGridColor`,        ['gray'], grid line color
+        - `xyAlpha`,              [0.15], grid plane opacity
+        - `xyFrameLine`,          [None], add a frame for the plane
+        - `showTicks`,            [True], show major ticks
+        - `digits`,               [None], use this number of significant digits in scientific notation
+        - `titleFont`,              [''], font for axes titles
+        - `labelFont`,              [''], font for numeric labels
+        - `textScale`,             [1.0], global scaling factor for text elements (titles, labels)
+        - `xTitlePosition`,       [0.32], title fractional positions along axis
+        - `xTitleOffset`,         [0.05], title fractional offset distance from axis line
         - `xTitleJustify`, ["top-right"], title justification
-        - `xTitleRotation`,      [0], add a rotation of the axis title
-        - `xLineColor`,  [automatic], color of the x-axis
-        - `xTitleColor`, [automatic], color of the axis title
-        - `xTitleBackfaceColor`, [None],  color of axis title on its backface
-        - `xTitleSize`,      [0.025], size of the axis title
-        - `xHighlightZero`,   [True], draw a line highlighting zero position if in range
-        - `xHighlightZeroColor`, [automatic], color of the line highlighting the zero position
-        - `xTickRadius`,     [0.005], radius of the major ticks
-        - `xTickThickness`, [0.0025], thickness of the major ticks along their axis
-        - `xTickColor`,  [automatic], color of major ticks
-        - `xMinorTicks`,         [1], number of minor ticks between two major ticks
-        - `tipSize`,          [0.01], size of the arrow tip
-        - `xPositionsAndLabels`   [], assign custom tick positions and labels [(pos1, label1), ...]
-        - `xLabelPrecision`,     [2], nr. of significative digits to be shown
-        - `xLabelSize`,      [0.015], size of the numeric labels along axis
-        - `xLabelOffset`,    [0.025], offset of numeric labels
-        - `limitRatio`,       [0.04], below this ratio don't plot small axis
+        - `xTitleRotation`,          [0], add a rotation of the axis title
+        - `xTitleBox`,           [False], add a box around title text
+        - `xLineColor`,      [automatic], color of the x-axis
+        - `xTitleColor`,     [automatic], color of the axis title
+        - `xTitleBackfaceColor`,  [None],  color of axis title on its backface
+        - `xTitleSize`,          [0.025], size of the axis title
+        - 'xTitleItalic',            [0], a bool or float to make the font italic
+        - `xHighlightZero`,       [True], draw a line highlighting zero position if in range
+        - `xHighlightZeroColor`, [autom], color of the line highlighting the zero position
+        - `xTickLength`,         [0.005], radius of the major ticks
+        - `xTickThickness`,     [0.0025], thickness of the major ticks along their axis
+        - `xMinorTicks`,             [1], number of minor ticks between two major ticks
+        - `xValuesAndLabels`          [], assign custom tick positions and labels [(pos1, label1), ...]
+        - `xLabelColor`,     [automatic], color of numeric labels and ticks
+        - `xLabelPrecision`,         [2], nr. of significative digits to be shown
+        - `xLabelSize`,          [0.015], size of the numeric labels along axis
+        - 'xLabelRotation',          [0], rotate clockwise [1] or anticlockwise [-1] by 90 degrees
+        - 'xFlipText',           [False], flip axis title and numeric labels orientation
+        - `xLabelOffset`,        [0.025], offset of numeric labels
+        - `tipSize`,              [0.01], size of the arrow tip
+        - `limitRatio`,           [0.04], below this ratio don't plot small axis
 
         :Example:
 
             .. code-block:: python
 
                 from vedo import Box, show
-
                 b = Box(pos=(1,2,3), length=8, width=9, height=7).alpha(0)
-                bax = b.buildAxes(c='white')  # returns Assembly object
-
+                bax = b.buildAxes(c='k')  # returns Assembly object
                 show(b, bax)
 
         |customAxes| |customAxes.py|_
@@ -476,6 +479,13 @@ class Base3DProp(object):
         self.axes = a
         return a
 
+#    def delete(self):
+#        """
+#        """
+#        settings.collectable_actors.remove(self)
+#        del self
+
+
     def show(self, **options):
         """
         Create on the fly an instance of class ``Plotter`` or use the last existing one to
@@ -515,9 +525,12 @@ class BaseActor(Base3DProp):
         Base3DProp.__init__(self)
 
         self.scalarbar = None
-        self.flagText = None
         self._mapper = None
 
+        self.flagText = None
+        self._caption = None
+
+
     def mapper(self, newMapper=None):
         """Return the ``vtkMapper`` data object, or update it with a new one."""
         if newMapper:
@@ -555,7 +568,10 @@ class BaseActor(Base3DProp):
         |delaunay2d| |delaunay2d.py|_
         """
         vcen = vtk.vtkCellCenters()
-        vcen.SetInputData(self.inputdata())
+        if hasattr(self, "polydata"):
+            vcen.SetInputData(self.polydata())
+        else:
+            vcen.SetInputData(self.inputdata())
         vcen.Update()
         return vtk_to_numpy(vcen.GetOutput().GetPoints().GetData())
 
@@ -606,20 +622,6 @@ class BaseActor(Base3DProp):
         return np.array(cids)
 
 
-    def flag(self, text=None):
-        """Add a flag label which becomes visible when hovering the object with mouse.
-        Can be later disabled by setting `flag(False)`.
-        """
-        if text is None:
-            if self.filename:
-                text = self.filename.split('/')[-1]
-            elif self.name:
-                text = self.name
-            else:
-                text = ""
-        self.flagText = text
-        return self
-
     def lighting(self, style='', ambient=None, diffuse=None,
                  specular=None, specularPower=None, specularColor=None):
         """
@@ -659,8 +661,8 @@ class BaseActor(Base3DProp):
             elif style=='ambient' : pars = [0.8, 0.1, 0.0,  0, (1,1,1)]
             elif style=='default' : pars = [0.1, 1.0, 0.05, 5, c]
             else:
-                colors.printc("Error in lighting(): Available styles are", c=1)
-                colors.printc("[default,metallic,plastic,shiny,glossy,ambient,off]", c=1)
+                colors.printc("Error in lighting(): Available styles are", c='r')
+                colors.printc("[default,metallic,plastic,shiny,glossy,ambient,off]", c='r')
                 raise RuntimeError()
             pr.SetAmbient(pars[0])
             pr.SetDiffuse(pars[1])
@@ -851,7 +853,7 @@ class BaseActor(Base3DProp):
 
         if len(input_array) != data.GetNumberOfPoints():
             colors.printc('Error in addPointArray(): Number of inputs != nr. of points',
-                          len(input_array), data.GetNumberOfPoints(), c=1)
+                          len(input_array), data.GetNumberOfPoints(), c='r')
             raise RuntimeError()
 
         nparr = np.ascontiguousarray(input_array)
@@ -876,7 +878,7 @@ class BaseActor(Base3DProp):
                 data.GetPointData().SetActiveVectors(name)
         else:
             colors.printc('Error in addPointArray(): cannot deal with shape:',
-                          nparr.shape, c=1)
+                          nparr.shape, c='r')
             return self
 
         if hasattr(self._mapper, 'SetArrayName'):
@@ -905,7 +907,7 @@ class BaseActor(Base3DProp):
 
         if len(input_array) != data.GetNumberOfCells():
             colors.printc('Error in addCellArray(): Number of inputs != nr. of Cells',
-                          len(input_array), data.GetNumberOfCells(), c=1)
+                          len(input_array), data.GetNumberOfCells(), c='r')
             raise RuntimeError()
 
         nparr = np.ascontiguousarray(input_array)
@@ -929,7 +931,7 @@ class BaseActor(Base3DProp):
                 data.GetCellData().SetActiveVectors(name)
         else:
             colors.printc('Error in addCellArray(): cannot deal with shape:',
-                          nparr.shape, c=1)
+                          nparr.shape, c='r')
             return self
 
         if hasattr(self._mapper, 'SetArrayName'):
@@ -956,6 +958,39 @@ class BaseActor(Base3DProp):
         return self.addCellArray(vectors, name)
 
 
+    def gradient(self, arrname=None, on='points'):
+        """
+        Compute and return the gradiend of a scalar field as a numpy array.
+
+        :param str arrname: name of the existing scalar field
+        :param str on: either 'points' or 'cells'
+
+        |isolines| |isolines.py|_
+        """
+        gra = vtk.vtkGradientFilter()
+        if on.startswith('p'):
+            varr = self.inputdata().GetPointData()
+            tp = vtk.vtkDataObject.FIELD_ASSOCIATION_POINTS
+        else:
+            varr = self.inputdata().GetCellData()
+            tp = vtk.vtkDataObject.FIELD_ASSOCIATION_CELLS
+        if not arrname:
+            if self.GetScalars():
+                arrname = varr.GetScalars().GetName()
+            else:
+                colors.printc('Error in gradient: no scalars found for', on, c='r')
+                raise RuntimeError
+        gra.SetInputData(self.inputdata())
+        gra.SetInputScalars(tp, arrname)
+        gra.SetResultArrayName('Gradients')
+        gra.Update()
+        if on.startswith('p'):
+            gvecs = vtk_to_numpy(gra.GetOutput().GetPointData().GetArray('Gradients'))
+        else:
+            gvecs = vtk_to_numpy(gra.GetOutput().GetCellData().GetArray('Gradients'))
+        return gvecs
+
+
     def mapCellsToPoints(self):
         """
         Transform cell data (i.e., data specified per cell)
@@ -1021,11 +1056,13 @@ class BaseActor(Base3DProp):
         sx=None,
         sy=None,
         title='',
+        titleFont="",
         titleXOffset = -1.5,
         titleYOffset = 0.0,
         titleSize =  1.5,
         titleRotation = 0.0,
         nlabels=9,
+        labelFont="",
         labelOffset = 0.375,
         italic=0,
         c=None,
@@ -1064,11 +1101,13 @@ class BaseActor(Base3DProp):
                                                 sx,
                                                 sy,
                                                 title,
+                                                titleFont,
                                                 titleXOffset,
                                                 titleYOffset,
                                                 titleSize,
                                                 titleRotation,
                                                 nlabels,
+                                                labelFont,
                                                 labelOffset,
                                                 italic,
                                                 c,
@@ -1080,13 +1119,13 @@ class BaseActor(Base3DProp):
 
     def write(self, filename, binary=True):
         """Write object to file."""
-        import vedo.vtkio as vtkio
-        return vtkio.write(self, filename, binary)
+        import vedo.io as io
+        return io.write(self, filename, binary)
 
 
+########################################################################################
 class BaseGrid(BaseActor):
-    """
-    """
+
     def __init__(self):
 
         BaseActor.__init__(self)
@@ -1239,7 +1278,7 @@ class BaseGrid(BaseActor):
             ctf.AddRGBPoint(smin, r,g,b) # constant color
             ctf.AddRGBPoint(smax, r,g,b)
         else:
-            colors.printc("volume.color(): unknown input type:", col, c=1)
+            colors.printc("ugrid.color(): unknown input type:", col, c='r')
         return self
 
     def alpha(self, alpha):
@@ -1308,11 +1347,11 @@ class BaseGrid(BaseActor):
         sf.Update()
         return self._update(sf.GetOutput())
 
-    def isosurface(self, threshold=True, connectivity=False):
+    def isosurface(self, threshold=None, largest=False):
         """Return an ``Mesh`` isosurface extracted from the ``Volume`` object.
 
         :param float,list threshold: value or list of values to draw the isosurface(s)
-        :param bool connectivity: if True only keeps the largest portion of the polydata
+        :param bool largest: if True keep only the largest portion of the mesh
 
         |isosurfaces| |isosurfaces.py|_
         """
@@ -1321,28 +1360,21 @@ class BaseGrid(BaseActor):
         cf = vtk.vtkContourFilter()
         cf.SetInputData(self._data)
         cf.UseScalarTreeOn()
-        cf.ComputeScalarsOn()
         cf.ComputeNormalsOn()
 
         if utils.isSequence(threshold):
             cf.SetNumberOfContours(len(threshold))
             for i, t in enumerate(threshold):
                 cf.SetValue(i, t)
-            cf.Update()
         else:
-            if threshold is True:
+            if threshold is None:
                 threshold = (2 * scrange[0] + scrange[1]) / 3.0
-                #print('automatic threshold set to ' + utils.precision(threshold, 3), end=' ')
-                #print('in [' + utils.precision(scrange[0], 3) + ', ' + utils.precision(scrange[1], 3)+']')
             cf.SetValue(0, threshold)
-            cf.Update()
 
-        clp = vtk.vtkCleanPolyData()
-        clp.SetInputConnection(cf.GetOutputPort())
-        clp.Update()
-        poly = clp.GetOutput()
+        cf.Update()
+        poly = cf.GetOutput()
 
-        if connectivity:
+        if largest:
             conn = vtk.vtkPolyDataConnectivityFilter()
             conn.SetExtractionModeToLargestRegion()
             conn.SetInputData(poly)
@@ -1398,7 +1430,7 @@ class BaseGrid(BaseActor):
         if scalars is None:
             print("Error in legosurface(): no scalars found!")
             return a
-        a.pointColors(scalars, vmin=srng[0], vmax=srng[1], cmap=cmap)
+        a.cmap(cmap, scalars, vmin=srng[0], vmax=srng[1])
         a.mapPointsToCells()
         return a
 
@@ -1561,6 +1593,367 @@ class BaseGrid(BaseActor):
 
 
 
+############################################################################### funcs
+def _getinput(obj):
+    if isinstance(obj, (vtk.vtkVolume, vtk.vtkActor)):
+        return obj.GetMapper().GetInput()
+    else:
+        return obj
+
+def probePoints(dataset, pts):
+    """
+    Takes a ``Volume`` (or any other vtk data set)
+    and probes its scalars at the specified points in space.
+
+    Note that a mask is also output with valid/invalid points which can be accessed
+    with `mesh.getPointArray('vtkValidPointMask')`.
+    """
+    if isinstance(pts, vedo.pointcloud.Points):
+        pts = pts.points()
+
+    def readPoints():
+        output = src.GetPolyDataOutput()
+        points = vtk.vtkPoints()
+        for p in pts:
+            x, y, z = p
+            points.InsertNextPoint(x, y, z)
+        output.SetPoints(points)
+
+        cells = vtk.vtkCellArray()
+        cells.InsertNextCell(len(pts))
+        for i in range(len(pts)):
+            cells.InsertCellPoint(i)
+        output.SetVerts(cells)
+
+    src = vtk.vtkProgrammableSource()
+    src.SetExecuteMethod(readPoints)
+    src.Update()
+    img = _getinput(dataset)
+    probeFilter = vtk.vtkProbeFilter()
+    probeFilter.SetSourceData(img)
+    probeFilter.SetInputConnection(src.GetOutputPort())
+    probeFilter.Update()
+    poly = probeFilter.GetOutput()
+    pm = vedo.mesh.Mesh(poly)
+    pm.name = 'probePoints'
+    return pm
+
+def probeLine(dataset, p1, p2, res=100):
+    """
+    Takes a ``Volume``  (or any other vtk data set)
+    and probes its scalars along a line defined by 2 points `p1` and `p2`.
+
+    Note that a mask is also output with valid/invalid points which can be accessed
+    with `mesh.getPointArray('vtkValidPointMask')`.
+
+    :param int res: nr of points along the line
+
+    |probeLine1| |probeLine1.py|_ |probeLine2.py|_
+    """
+    line = vtk.vtkLineSource()
+    line.SetResolution(res)
+    line.SetPoint1(p1)
+    line.SetPoint2(p2)
+    img = _getinput(dataset)
+    probeFilter = vtk.vtkProbeFilter()
+    probeFilter.SetSourceData(img)
+    probeFilter.SetInputConnection(line.GetOutputPort())
+    probeFilter.Update()
+    poly = probeFilter.GetOutput()
+    lnn = vedo.mesh.Mesh(poly)
+    lnn.name = 'probeLine'
+    return lnn
+
+def probePlane(dataset, origin=(0, 0, 0), normal=(1, 0, 0)):
+    """
+    Takes a ``Volume`` (or any other vtk data set)
+    and probes its scalars on a plane defined by a point and a normal.
+    """
+    img = _getinput(dataset)
+    plane = vtk.vtkPlane()
+    plane.SetOrigin(origin)
+    plane.SetNormal(normal)
+    planeCut = vtk.vtkCutter()
+    planeCut.SetInputData(img)
+    planeCut.SetCutFunction(plane)
+    planeCut.Update()
+    poly = planeCut.GetOutput()
+    cutmesh = vedo.mesh.Mesh(poly)
+    cutmesh.name = 'probePlane'
+    return cutmesh
+
+
+def interpolateToStructuredGrid(mesh, kernel=None, radius=None,
+                               bounds=None, nullValue=None, dims=None):
+    """
+    Generate a volumetric dataset (vtkStructuredData) by interpolating a scalar
+    or vector field which is only known on a scattered set of points or mesh.
+    Available interpolation kernels are: shepard, gaussian, voronoi, linear.
+
+    :param str kernel: interpolation kernel type [shepard]
+    :param float radius: radius of the local search
+    :param list bounds: bounding box of the output vtkStructuredGrid object
+    :param list dims: dimensions of the output vtkStructuredGrid object
+    :param float nullValue: value to be assigned to invalid points
+    """
+    if isinstance(mesh, vtk.vtkPolyData):
+        output = mesh
+    else:
+        output = mesh.polydata()
+
+    if dims is None:
+        dims = (20,20,20)
+
+    if bounds is None:
+        bounds = output.GetBounds()
+
+    # Create a probe volume
+    probe = vtk.vtkStructuredGrid()
+    probe.SetDimensions(dims)
+
+    points = vtk.vtkPoints()
+    points.Allocate(dims[0] * dims[1] * dims[2])
+    deltaZ = (bounds[5]-bounds[4]) / (dims[2] - 1)
+    deltaY = (bounds[3]-bounds[2]) / (dims[1] - 1)
+    deltaX = (bounds[1]-bounds[0]) / (dims[0] - 1)
+    for k in range(dims[2]):
+        z = bounds[4] + k * deltaZ
+        kOffset = k * dims[0] * dims[1]
+        for j in range(dims[1]):
+            y = bounds[2] + j * deltaY
+            jOffset = j * dims[0]
+            for i  in range(dims[0]):
+                x = bounds[0] + i * deltaX
+                offset = i + jOffset + kOffset
+                points.InsertPoint(offset, [x,y,z])
+    probe.SetPoints(points)
+
+    if radius is None:
+        radius = min(bounds[1]-bounds[0], bounds[3]-bounds[2], bounds[5]-bounds[4])/3
+
+    locator = vtk.vtkPointLocator()
+    locator.SetDataSet(output)
+    locator.BuildLocator()
+
+    if kernel == 'gaussian':
+        kern = vtk.vtkGaussianKernel()
+        kern.SetRadius(radius)
+    elif kernel == 'voronoi':
+        kern = vtk.vtkVoronoiKernel()
+    elif kernel == 'linear':
+        kern = vtk.vtkLinearKernel()
+        kern.SetRadius(radius)
+    else:
+        kern = vtk.vtkShepardKernel()
+        kern.SetPowerParameter(2)
+        kern.SetRadius(radius)
+
+    interpolator = vtk.vtkPointInterpolator()
+    interpolator.SetInputData(probe)
+    interpolator.SetSourceData(output)
+    interpolator.SetKernel(kern)
+    interpolator.SetLocator(locator)
+    if nullValue is not None:
+        interpolator.SetNullValue(nullValue)
+    else:
+        interpolator.SetNullPointsStrategyToClosestPoint()
+    interpolator.Update()
+    return interpolator.GetOutput()
+
+
+def streamLines(domain, probe,
+                activeVectors='',
+                integrator='rk4',
+                direction='forward',
+                initialStepSize=None,
+                maxPropagation=None,
+                maxSteps=10000,
+                stepLength=None,
+                extrapolateToBoundingBox=(),
+                surfaceConstrain=False,
+                computeVorticity=True,
+                ribbons=None,
+                tubes={},
+                scalarRange=None,
+                lw=None,
+    ):
+    """
+    Integrate a vector field on a domain (a Mesh or other vtk datasets types)
+    to generate streamlines.
+
+    The integration is performed using a specified integrator (Runge-Kutta).
+    The length of a streamline is governed by specifying a maximum value either
+    in physical arc length or in (local) cell length.
+    Otherwise, the integration terminates upon exiting the field domain.
+
+    :param domain: the vtk object that contains the vector field
+    :param str activeVectors: name of the vector array
+    :param Mesh,list probe: the Mesh that probes the domain. Its coordinates will
+        be the seeds for the streamlines, can also be an array of positions.
+    :param str integrator: Runge-Kutta integrator, either 'rk2', 'rk4' of 'rk45'
+    :param float initialStepSize: initial step size of integration
+    :param float maxPropagation: maximum physical length of the streamline
+    :param int maxSteps: maximum nr of steps allowed
+    :param float stepLength: length of step integration.
+    :param dict extrapolateToBoundingBox:
+        Vectors defined on a surface are extrapolated to the entire volume defined by its bounding box
+
+        - kernel, (str) - interpolation kernel type [shepard]
+        - radius (float)- radius of the local search
+        - bounds, (list) - bounding box of the output Volume
+        - dims, (list) - dimensions of the output Volume object
+        - nullValue, (float) - value to be assigned to invalid points
+
+    :param bool surfaceConstrain: force streamlines to be computed on a surface
+    :param bool computeVorticity: Turn on/off vorticity computation at streamline points
+        (necessary for generating proper stream-ribbons)
+    :param int ribbons: render lines as ribbons by joining them.
+        An integer value represent the ratio of joining (e.g.: ribbons=2 groups lines 2 by 2)
+    :param dict tubes: dictionary containing the parameters for the tube representation:
+
+            - ratio, (int) - draws tube as longitudinal stripes
+            - res, (int) - tube resolution (nr. of sides, 12 by default)
+            - maxRadiusFactor (float) - max tube radius as a multiple of the min radius
+            - varyRadius, (int) - radius varies based on the scalar or vector magnitude:
+
+                - 0 - do not vary radius
+                - 1 - vary radius by scalar
+                - 2 - vary radius by vector
+                - 3 - vary radius by absolute value of scalar
+
+    :param list scalarRange: specify the scalar range for coloring
+
+    .. hint:: Examples: |streamlines1.py|_ |streamribbons.py|_ |office.py|_ |streamlines2.py|_
+
+        |streamlines2| |office| |streamribbons| |streamlines1|
+    """
+
+    if isinstance(domain, vtk.vtkActor):
+        if len(extrapolateToBoundingBox):
+            grid = interpolateToStructuredGrid(domain, **extrapolateToBoundingBox)
+        else:
+            grid = domain.polydata()
+    else:
+        grid = domain
+
+    if activeVectors:
+        grid.GetPointData().SetActiveVectors(activeVectors)
+
+    b = grid.GetBounds()
+    size = (b[5]-b[4] + b[3]-b[2] + b[1]-b[0])/3
+    if initialStepSize is None:
+        initialStepSize = size/100.
+    if maxPropagation is None:
+        maxPropagation = size
+
+    if utils.isSequence(probe):
+        pts = np.array(probe)
+        if pts.shape[1] == 2: # make it 3d
+            pts = np.c_[pts, np.zeros(len(pts))]
+    else:
+        pts = probe.clean().points()
+    src = vtk.vtkProgrammableSource()
+    def readPoints():
+        output = src.GetPolyDataOutput()
+        points = vtk.vtkPoints()
+        for x, y, z in pts:
+            points.InsertNextPoint(x, y, z)
+        output.SetPoints(points)
+    src.SetExecuteMethod(readPoints)
+    src.Update()
+
+    st = vtk.vtkStreamTracer()
+    st.SetInputDataObject(grid)
+    st.SetSourceConnection(src.GetOutputPort())
+
+    st.SetInitialIntegrationStep(initialStepSize)
+    st.SetComputeVorticity(computeVorticity)
+    st.SetMaximumNumberOfSteps(maxSteps)
+    st.SetMaximumPropagation(maxPropagation)
+    st.SetSurfaceStreamlines(surfaceConstrain)
+    if stepLength:
+        st.SetMaximumIntegrationStep(stepLength)
+
+    if 'f' in direction:
+        st.SetIntegrationDirectionToForward()
+    elif 'back' in direction:
+        st.SetIntegrationDirectionToBackward()
+    elif 'both' in direction:
+        st.SetIntegrationDirectionToBoth()
+
+    if integrator == 'rk2':
+        st.SetIntegratorTypeToRungeKutta2()
+    elif integrator == 'rk4':
+        st.SetIntegratorTypeToRungeKutta4()
+    elif integrator == 'rk45':
+        st.SetIntegratorTypeToRungeKutta45()
+    else:
+        colors.printc("Error in streamlines, unknown integrator", integrator, c='r')
+
+    st.Update()
+    output = st.GetOutput()
+
+    if ribbons:
+        scalarSurface = vtk.vtkRuledSurfaceFilter()
+        scalarSurface.SetInputConnection(st.GetOutputPort())
+        scalarSurface.SetOnRatio(int(ribbons))
+        scalarSurface.SetRuledModeToPointWalk()
+        scalarSurface.Update()
+        output = scalarSurface.GetOutput()
+
+    if len(tubes):
+        streamTube = vtk.vtkTubeFilter()
+        streamTube.SetNumberOfSides(12)
+        streamTube.SetRadius(tubes['radius'])
+
+        if 'res' in tubes:
+            streamTube.SetNumberOfSides(tubes['res'])
+
+        # max tube radius as a multiple of the min radius
+        streamTube.SetRadiusFactor(50)
+        if 'maxRadiusFactor' in tubes:
+            streamTube.SetRadius(tubes['maxRadiusFactor'])
+
+        if 'ratio' in tubes:
+            streamTube.SetOnRatio(int(tubes['ratio']))
+
+        if 'varyRadius' in tubes:
+            streamTube.SetVaryRadius(int(tubes['varyRadius']))
+
+        streamTube.SetInputData(output)
+        vname = grid.GetPointData().GetVectors().GetName()
+        streamTube.SetInputArrayToProcess(1, 0, 0,
+                                          vtk.vtkDataObject.FIELD_ASSOCIATION_POINTS,
+                                          vname)
+        streamTube.Update()
+        sta = vedo.mesh.Mesh(streamTube.GetOutput(), c=None)
+
+        scals = grid.GetPointData().GetScalars()
+        if scals:
+            sta.mapper().SetScalarRange(scals.GetRange())
+        if scalarRange is not None:
+            sta.mapper().SetScalarRange(scalarRange)
+
+        sta.GetProperty().BackfaceCullingOn()
+        sta.phong()
+        return sta
+
+    sta = vedo.mesh.Mesh(output, c=None)
+
+    if lw is not None and len(tubes)==0 and not ribbons:
+        sta.lw(lw)
+        sta._mapper.SetResolveCoincidentTopologyToPolygonOffset()
+        sta.lighting('off')
+
+    scals = grid.GetPointData().GetScalars()
+    if scals:
+        sta.mapper().SetScalarRange(scals.GetRange())
+    if scalarRange is not None:
+        sta.mapper().SetScalarRange(scalarRange)
+    return sta
+
+
+
 ###################################################################################
 # def extractCellsByType(obj, types=(7,)):    ### VTK9 only
 #     """Extract cells of a specified type.
@@ -1574,5 +1967,3 @@ class BaseGrid(BaseActor):
 #         ef.AddCellType(ct)
 #     ef.Update()
 #     return Mesh(ef.GetOutput())
-
-
