@@ -17,7 +17,7 @@ from vedo.settings import datadir, embedWindow
 
 from vedo.mesh import Mesh
 
-from vedo.io import load, screenshot, Video, exportWindow
+from vedo.io import load, screenshot, Video, exportWindow, download
 
 import vedo.shapes as shapes
 from vedo.shapes import Text, Text2D, Latex
@@ -25,7 +25,7 @@ from vedo.shapes import Text, Text2D, Latex
 from vedo.plotter import show, clear, Plotter
 from vedo.plotter import closeWindow, closePlotter, interactive
 
-from vedo.pyplot import histogram, plotMatrix
+from vedo.pyplot import histogram
 
 # Install fenics with commands (e.g. in Anaconda3):
 #         conda install -c conda-forge fenics
@@ -107,6 +107,7 @@ __all__ = [
     "plot",
     "histogram",
     "load",
+    "download",
     "show",
     "clear",
     "printc",
@@ -119,7 +120,6 @@ __all__ = [
     "datadir",
     "screenshot",
     "Video",
-    "plotMatrix",
     "exportWindow",
     "closeWindow",
     "closePlotter",
@@ -208,9 +208,9 @@ def _inputsort(obj):
                 mesh = ob.mesh()
 
                 if ob.dim()>0:
-                    printc('MeshFunction of dim>0 not supported.', c=1)
-                    printc('Try e.g.:  MeshFunction("size_t", mesh, 0)', c=1, italic=1)
-                    printc('instead of MeshFunction("size_t", mesh, 1)', c=1, strike=1)
+                    printc('MeshFunction of dim>0 not supported.', c='r')
+                    printc('Try e.g.:  MeshFunction("size_t", mesh, 0)', c='r', italic=1)
+                    printc('instead of MeshFunction("size_t", mesh, 1)', c='r', strike=1)
                 else:
                     #printc(ob.dim(), mesh.num_cells(), len(mesh.coordinates()), len(ob.array()))
                     V = dolfin.FunctionSpace(mesh, "CG", 1)
@@ -297,7 +297,7 @@ def _compute_uvalues(u, mesh):
         coords = mesh.geometry.points
 
     if u_values.shape[0] != coords.shape[0]:
-        printc('Warning: mismatch in vedo.dolfin._compute_uvalues()', c=1)
+        printc('Warning: mismatch in vedo.dolfin._compute_uvalues()', c='r')
         u_values = np.array([u(p) for p in coords])
     return u_values
 
@@ -360,7 +360,7 @@ def plot(*inputobj, **options):
 
     :param list scaleMeshFactors: rescale mesh by these factors [1,1,1]
 
-    :param bool newPlotter: spawn a new instance of Plotter class, pops up a new window
+    :param bool new: spawn a new instance of Plotter class, pops up a new window
     :param int at: renderer number to plot to
     :param list shape: subdvide window in (n,m) rows and columns
     :param int N: automatically subdvide window in N renderers
@@ -380,19 +380,19 @@ def plot(*inputobj, **options):
 
     :param int axes: axes type number
 
-      - 0,  no axes,
-      - 1,  draw customizable grid axes (see below).
-      - 2,  show cartesian axes from (0,0,0)
-      - 3,  show positive range of cartesian axes from (0,0,0)
-      - 4,  show a triad at bottom left
-      - 5,  show a cube at bottom left
-      - 6,  mark the corners of the bounding box
-      - 7,  draw a simple ruler at the bottom of the window
-      - 8,  show the `vtkCubeAxesActor` object,
-      - 9,  show the bounding box outLine,
-      - 10, show three circles representing the maximum bounding box,
-      - 11, show a large grid on the x-y plane (use with zoom=8)
-      - 12, show polar axes.
+       - 0,  no axes,
+       - 1,  draw customizable grid axes (see below).
+       - 2,  show cartesian axes from (0,0,0)
+       - 3,  show positive range of cartesian axes from (0,0,0)
+       - 4,  show a triad at bottom left
+       - 5,  show a cube at bottom left
+       - 6,  mark the corners of the bounding box
+       - 7,  draw a simple ruler at the bottom of the window
+       - 8,  show the `vtkCubeAxesActor` object,
+       - 9,  show the bounding box outLine,
+       - 10, show three circles representing the maximum bounding box,
+       - 11, show a large grid on the x-y plane (use with zoom=8)
+       - 12, show polar axes.
 
     Axes type-1 can be fully customized by passing a dictionary ``axes=dict()`` where:
 
@@ -402,7 +402,6 @@ def plot(*inputobj, **options):
         - `axesLineWidth`,           [1], width of the axes lines
         - `gridLineWidth`,           [1], width of the grid lines
         - `reorientShortTitle`,   [True], titles shorter than 2 letter are placed horizontally
-        - `originMarkerSize`,     [0.01], draw a small cube on the axis where the origin is
         - `titleDepth`,              [0], extrusion fractional depth of title text
         - `xyGrid`,               [True], show a gridded wall on plane xy
         - `yzGrid`,               [True], show a gridded wall on plane yz
@@ -415,10 +414,15 @@ def plot(*inputobj, **options):
         - `xyAlpha`,              [0.15], grid plane opacity
         - `xyFrameLine`,          [None], add a frame for the plane
         - `showTicks`,            [True], show major ticks
+        - `digits`,               [None], use this number of significant digits in scientific notation
+        - `titleFont`,              [''], font for axes titles
+        - `labelFont`,              [''], font for numeric labels
+        - `textScale`,             [1.0], global scaling factor for text elements (titles, labels)
         - `xTitlePosition`,       [0.32], title fractional positions along axis
         - `xTitleOffset`,         [0.05], title fractional offset distance from axis line
         - `xTitleJustify`, ["top-right"], title justification
         - `xTitleRotation`,          [0], add a rotation of the axis title
+        - `xTitleBox`,           [False], add a box around title text
         - `xLineColor`,      [automatic], color of the x-axis
         - `xTitleColor`,     [automatic], color of the axis title
         - `xTitleBackfaceColor`,  [None],  color of axis title on its backface
@@ -428,13 +432,14 @@ def plot(*inputobj, **options):
         - `xHighlightZeroColor`, [autom], color of the line highlighting the zero position
         - `xTickLength`,         [0.005], radius of the major ticks
         - `xTickThickness`,     [0.0025], thickness of the major ticks along their axis
-        - `xTickColor`,      [automatic], color of major ticks
         - `xMinorTicks`,             [1], number of minor ticks between two major ticks
-        - `xPositionsAndLabels`       [], assign custom tick positions and labels [(pos1, label1), ...]
+        - `xValuesAndLabels`          [], assign custom tick positions and labels [(pos1, label1), ...]
+        - `xLabelColor`,     [automatic], color of numeric labels and ticks
         - `xLabelPrecision`,         [2], nr. of significative digits to be shown
         - `xLabelSize`,          [0.015], size of the numeric labels along axis
+        - 'xLabelRotation',          [0], rotate clockwise [1] or anticlockwise [-1] by 90 degrees
+        - 'xFlipText',           [False], flip axis title and numeric labels orientation
         - `xLabelOffset`,        [0.025], offset of numeric labels
-        - 'xFlipText'.           [False], flip axis title and numeric labels orientation
         - `tipSize`,              [0.01], size of the arrow tip
         - `limitRatio`,           [0.04], below this ratio don't plot small axis
 
@@ -691,11 +696,9 @@ def plot(*inputobj, **options):
 
         if cmap and (actor.u_values is not None) and c is None:
             if u.value_rank() > 0: # will show the size of the vector
-                actor.pointColors(utils.mag(actor.u_values),
-                                  vmin=vmin, vmax=vmax, cmap=cmap)
+                actor.cmap(cmap, utils.mag(actor.u_values), vmin=vmin, vmax=vmax)
             else:
-                actor.pointColors(actor.u_values,
-                                  vmin=vmin, vmax=vmax, cmap=cmap)
+                actor.cmap(cmap, actor.u_values, vmin=vmin, vmax=vmax)
 
         if warpYfactor:
             scals = actor.getPointArray()
@@ -862,8 +865,8 @@ class MeshActor(Mesh):
 
         if coords.shape != deltas.shape:
             printc("ERROR: Try to move mesh with wrong solution type shape:",
-                  coords.shape, 'vs', deltas.shape, c=1)
-            printc("Mesh is not moved. Try mode='color' in plot().", c=1)
+                  coords.shape, 'vs', deltas.shape, c='r')
+            printc("Mesh is not moved. Try mode='color' in plot().", c='r')
             return
 
         movedpts = coords + deltas
@@ -944,7 +947,7 @@ def MeshLines(*inputobj, **options):
 
     u_values = _compute_uvalues(u,mesh)
     if not utils.isSequence(u_values[0]):
-        printc("~times Error: cannot show Lines for 1D scalar values!", c=1)
+        printc("\times Error: cannot show Lines for 1D scalar values!", c='r')
         raise RuntimeError()
 
     endPoints = startPoints + u_values
@@ -985,7 +988,7 @@ def MeshArrows(*inputobj, **options):
 
     u_values = _compute_uvalues(u,mesh)
     if not utils.isSequence(u_values[0]):
-        printc("~times Error: cannot show Arrows for 1D scalar values!", c=1)
+        printc("\times Error: cannot show Arrows for 1D scalar values!", c='r')
         raise RuntimeError()
 
     endPoints = startPoints + u_values
@@ -1007,7 +1010,7 @@ def MeshStreamLines(*inputobj, **options):
     """
     Build streamplot.
     """
-    from vedo.analysis import streamLines
+    from vedo.base import streamLines
 
     print('Building streamlines...')
 
@@ -1028,7 +1031,7 @@ def MeshStreamLines(*inputobj, **options):
 
     u_values = _compute_uvalues(u, mesh)
     if not utils.isSequence(u_values[0]):
-        printc("~times Error: cannot show Arrows for 1D scalar values!", c=1)
+        printc("\times Error: cannot show Arrows for 1D scalar values!", c='r')
         raise RuntimeError()
     if u_values.shape[1] == 2:  # u_values is 2D
         u_values = np.insert(u_values, 2, 0, axis=1)  # make it 3d
