@@ -50,6 +50,10 @@ class Picture(vtk.vtkImageActor, Base3DProp):
             img = obj
 
         elif isinstance(obj, str):
+            if "https://" in obj:
+                import vedo.io as io
+                obj = io.download(obj, verbose=False)
+
             if   ".png" in obj:
                 picr = vtk.vtkPNGReader()
             elif ".jpg" in obj or ".jpeg" in obj:
@@ -59,7 +63,8 @@ class Picture(vtk.vtkImageActor, Base3DProp):
             elif ".tif" in obj:
                 picr = vtk.vtkTIFFReader()
             else:
-                colors.printc("Cannot understand picture format", obj, c=1)
+                colors.printc("Cannot understand picture format", obj, c='r')
+                return
             picr.SetFileName(obj)
             picr.Update()
             img = picr.GetOutput()
@@ -72,7 +77,7 @@ class Picture(vtk.vtkImageActor, Base3DProp):
         self._data = img
         self._mapper = self.GetMapper()
 
-    def imagedata(self):
+    def inputdata(self):
         """Return the underlying ``vtkImagaData`` object."""
         return self._data
 
@@ -133,8 +138,8 @@ class Picture(vtk.vtkImageActor, Base3DProp):
             elif os.path.exists(font):
                 tp.SetFontFile(font)
             else:
-                colors.printc("~sad Font", font, "not found in", settings.fonts_path, c="r")
-                colors.printc("~pin Available fonts are:", settings.fonts, c="m")
+                colors.printc("\sad Font", font, "not found in", settings.fonts_path, c="r")
+                colors.printc("\pin Available fonts are:", settings.fonts, c="m")
                 return None
 
         if bg:
@@ -203,7 +208,7 @@ class Picture(vtk.vtkImageActor, Base3DProp):
         elif axis.lower() == "y":
             ff.SetFilteredAxis(1)
         else:
-            colors.printc("~times Error in mirror(): mirror must be set to x or y.", c=1)
+            colors.printc("\times Error in mirror(): mirror must be set to x or y.", c='r')
             raise RuntimeError()
         ff.Update()
         return self._update(ff.GetOutput())
