@@ -11,6 +11,7 @@ import vedo.shapes as shapes
 import vedo.addons as addons
 from vedo.assembly import Assembly
 from vedo.mesh import Mesh, merge
+from vedo.pointcloud import Points
 from vedo.base import BaseActor
 
 __doc__ = """Plotting utility functions.""" + docs._defs
@@ -22,7 +23,6 @@ __all__ = [
     "quiver",
     "violin",
     "streamplot",
-    "plotMatrix",
     "DirectedGraph",
 ]
 
@@ -616,11 +616,11 @@ def _plotxy(
 
     if xerrors is not None and not errorBand:
         if len(xerrors) != len(data):
-            colors.printc("Error in plotxy(xerrors=...): mismatched array length.", c=1)
+            colors.printc("Error in plotxy(xerrors=...): mismatched array length.", c='r')
             return None
         errs = []
-        for i in range(len(data)):
-            xval, yval = data[i]
+        for i, dta in enumerate(data):
+            xval, yval = dta
             xerr = xerrors[i] / 2
             el = shapes.Line((xval - xerr, yval, offs), (xval + xerr, yval, offs))
             errs.append(el)
@@ -629,7 +629,7 @@ def _plotxy(
 
     if yerrors is not None and not errorBand:
         if len(yerrors) != len(data):
-            colors.printc("Error in plotxy(yerrors=...): mismatched array length.", c=1)
+            colors.printc("Error in plotxy(yerrors=...): mismatched array length.", c='r')
             return None
         errs = []
         for i in range(len(data)):
@@ -691,7 +691,7 @@ def _plotxy(
             labs.append([ynew, ts[i]])
         axes["xtitle"] = xtitle
         axes["ytitle"] = ytitle
-        axes["yPositionsAndLabels"] = labs
+        axes["yValuesAndLabels"] = labs
         axes["xrange"] = (x0lim, x1lim)
         axes["yrange"] = (y0lim, y1lim)
         axes["zrange"] = (0, 0)
@@ -751,7 +751,7 @@ def _plotFxy(
             exec(code, namespace)
             z = namespace["zfunc"]
         except:
-            colors.printc("Syntax Error in plotFxy()", c=1)
+            colors.printc("Syntax Error in plotFxy()", c='r')
             return None
 
     ps = vtk.vtkPlaneSource()
@@ -789,7 +789,7 @@ def _plotFxy(
         poly = cl.GetOutput()
 
     if not poly.GetNumberOfPoints():
-        colors.printc("Function is not real in the domain", c=1)
+        colors.printc("Function is not real in the domain", c='r')
         return None
 
     if zlim[0]:
@@ -875,7 +875,7 @@ def _plotFz(
             exec(code, namespace)
             z = namespace["zfunc"]
         except:
-            colors.printc("Syntax Error in complex plotFz()", c=1)
+            colors.printc("Syntax Error in complex plotFz()", c='r')
             return None
 
     ps = vtk.vtkPlaneSource()
@@ -900,8 +900,7 @@ def _plotFz(
 
     mesh = Mesh(poly, alpha).lighting("plastic")
     v = max(abs(np.min(arrImg)), abs(np.max(arrImg)))
-    mesh.pointColors(arrImg, cmap=cmap, vmin=-v, vmax=v)
-    # mesh.mapPointsToCells()
+    mesh.cmap(cmap, arrImg, vmin=-v, vmax=v)
     mesh.computeNormals().lw(lw)
 
     if zlimits[0]:
@@ -1093,7 +1092,7 @@ def _plotSpheric(rfunc, normalize=True, res=33, scalarbar=True, c="grey", alpha=
 
     ssurf.alpha(1).wireframe(0).lw(0.1)
 
-    ssurf.pointColors(newr, cmap=cmap)
+    ssurf.cmap(cmap, newr)
     ssurf.computeNormals()
 
     if scalarbar:
@@ -1233,7 +1232,7 @@ def _histogram1D(
             p0 = (myedges[i] + gap * binsize, 0, 0)
             p1 = (myedges[i + 1] - gap * binsize, fs[i], 0)
             r = shapes.Rectangle(p0, p1)
-            r.origin(p0)
+            r.origin(p0).PickableOff()
             maxheigth = max(maxheigth, p1[1])
             r.color(c).alpha(alpha).lighting('off').z(offs)
             rs.append(r)
@@ -1339,7 +1338,7 @@ def _histogram1D(
             labs.append([ynew, ts[i]])
         axes["xtitle"] = xtitle
         axes["ytitle"] = ytitle
-        axes["yPositionsAndLabels"] = labs
+        axes["yValuesAndLabels"] = labs
         axes["xrange"] = (x0lim, x1lim)
         axes["yrange"] = (y0lim, y1lim)
         axes["zrange"] = (0, 0)
@@ -1447,7 +1446,7 @@ def _histogram2D(
         resy=bins[1],
     )
     g.alpha(alpha).lw(lw).wireframe(0).flat().lighting('off')
-    g.cellColors(np.ravel(H.T), cmap=cmap)
+    g.cmap(cmap, np.ravel(H.T), on='cells')
     g.SetOrigin(x0lim, y0lim, 0)
     if scalarbar:
         sc = g.addScalarBar3D(c=bc)
@@ -1490,7 +1489,7 @@ def _histogram2D(
             labs.append([ynew, ts[i]])
         axes["xtitle"] = xtitle
         axes["ytitle"] = ytitle
-        axes["yPositionsAndLabels"] = labs
+        axes["yValuesAndLabels"] = labs
         axes["xrange"] = (x0lim, x1lim)
         axes["yrange"] = (y0lim, y1lim)
         axes["zrange"] = (0, 0)
@@ -1540,12 +1539,8 @@ def _histogramHexBin(
     alpha=1,
 ):
     if xtitle:
-        from vedo import settings
-
         settings.xtitle = xtitle
     if ytitle:
-        from vedo import settings
-
         settings.ytitle = ytitle
 
     xmin, xmax = np.min(xvalues), np.max(xvalues)
@@ -1608,8 +1603,7 @@ def _histogramHexBin(
             if c is None:
                 col = i
             h = Mesh(tf.GetOutput(), c=col, alpha=alpha).flat()
-            h.GetProperty().SetSpecular(0)
-            h.GetProperty().SetDiffuse(1)
+            h.lighting('plastic')
             h.PickableOff()
             hexs.append(h)
             if ne > binmax:
@@ -1777,7 +1771,8 @@ def _histogramPolar(
 
     for i,t in enumerate(thetas):
         if i < len(labels):
-            lab = shapes.Text(labels[i], (0, 0, 0), s=lsize, depth=0, justify="center")
+            lab = shapes.Text(labels[i], (0, 0, 0), #font="VTK",
+                              s=lsize, depth=0, justify="center")
             lab.pos(r2e *np.cos(t) * (1 + rgap) * lpos / 2,
                     r2e *np.sin(t) * (1 + rgap) * lpos / 2, 0.01)
             labs.append(lab)
@@ -1835,16 +1830,11 @@ def _histogramSpheric(
         newsgpts[fs] = np.c_[x, y, z]
 
     newsg.points(newsgpts)
-    newsg.cellColors(acounts, cmap=cmap)
+    newsg.cmap(cmap, acounts, on='cells')
 
     if scalarbar:
         newsg.addScalarBar()
     newsg.name = "histogramSpheric"
-    # from vedo.analysis import normalLines
-    # nrm = normalLines(newsg, scale=0.5)
-    # asse = Assembly(newsg, nrm)
-    # sg0 = shapes.Sphere(r=0.99, res=res, quads=True).lw(.1).c([.9,.9,.9])
-    # return Assembly(newsg, sg0)
     return newsg
 
 
@@ -1885,7 +1875,7 @@ def donut(
     angles = np.add.accumulate(2 * np.pi * fractions)
     angles[-1] = 2 * np.pi
     if angles[-2] > 2 * np.pi:
-        print("Error in donutPlot(): fractions must sum to 1.")
+        print("Error in donut(): fractions must sum to 1.")
         raise RuntimeError
 
     cols = []
@@ -1951,12 +1941,11 @@ def quiver(
     :param float headLength: fractional head length
     :param float headWidth: fractional head width
     :param bool fill: if False only generate the outline
-
     :param float scale: apply a rescaling factor to the length
 
     |quiver| |quiver.py|_
     """
-    if isinstance(points, Mesh):
+    if isinstance(points, Points):
         points = points.points()
     else:
         points = np.array(points)
@@ -2078,9 +2067,8 @@ def violin(
     return asse
 
 
-def streamplot(
-    X, Y, U, V, direction="both", maxPropagation=None, mode=1, lw=0.001, c=None, probes=[],
-):
+def streamplot(X, Y, U, V, direction="both",
+               maxPropagation=None, mode=1, lw=0.001, c=None, probes=()):
     """
     Generate a streamline plot of a vectorial field (U,V) defined at positions (X,Y).
     Returns a ``Mesh`` object.
@@ -2098,7 +2086,7 @@ def streamplot(
     |plot7_stream| |plot7_stream.py|_
     """
     from vedo.volume import Volume
-    from vedo.analysis import streamLines
+    from vedo.base import streamLines
 
     n = len(X)
     m = len(Y[0])
@@ -2108,8 +2096,6 @@ def streamplot(
 
     xmin, xmax = X[0][0], X[-1][-1]
     ymin, ymax = Y[0][0], Y[-1][-1]
-    # print('xrange:', xmin, xmax)
-    # print('yrange:', ymin, ymax)
 
     field = np.sqrt(U * U + V * V)
 
@@ -2123,7 +2109,7 @@ def streamplot(
     if len(probes) == 0:
         probe = shapes.Grid(pos=((n-1)/2,(n-1)/2,0), sx=n-1, sy=n-1, resx=n-1, resy=n-1)
     else:
-        if isinstance(probes, Mesh):
+        if isinstance(probes, Points):
             probes = probes.points()
         else:
             probes = np.array(probes)
@@ -2135,12 +2121,12 @@ def streamplot(
         probe = shapes.Points(probes)
 
     stream = streamLines(
-        vol.imagedata(),
-        probe,
-        tubes={"radius": lw, "varyRadius": mode,},
-        lw=lw,
-        maxPropagation=maxPropagation,
-        direction=direction,
+                            vol.imagedata(),
+                            probe,
+                            tubes={"radius": lw, "varyRadius": mode,},
+                            lw=lw,
+                            maxPropagation=maxPropagation,
+                            direction=direction,
     )
     if c is not None:
         stream.color(c)
@@ -2152,47 +2138,6 @@ def streamplot(
     stream.addPos(np.array([xmin, ymin, 0]))
     return stream
 
-def plotMatrix(M, title='matrix', continuous=True, cmap='Greys'):
-    """
-	 Plot a matrix using `matplotlib`.
-
-    :Example:
-        .. code-block:: python
-
-            from vedo.dolfin import plotMatrix
-            import numpy as np
-
-            M = np.eye(9) + np.random.randn(9,9)/4
-
-            plotMatrix(M)
-
-        |pmatrix|
-    """
-    import matplotlib.pyplot as plt
-    import matplotlib as mpl
-    from mpl_toolkits.axes_grid1 import make_axes_locatable
-
-    M    = np.array(M)
-    m,n  = np.shape(M)
-    M    = M.round(decimals=2)
-
-    fig  = plt.figure()
-    ax   = fig.add_subplot(111)
-    cmap = mpl.cm.get_cmap(cmap)
-    if not continuous:
-        unq  = np.unique(M)
-    im      = ax.imshow(M, cmap=cmap, interpolation='None')
-    divider = make_axes_locatable(ax)
-    cax     = divider.append_axes("right", size="5%", pad=0.05)
-    dim     = r'$%i \times %i$ ' % (m,n)
-    ax.set_title(dim + title)
-    ax.axis('off')
-    cb = plt.colorbar(im, cax=cax)
-    if not continuous:
-       cb.set_ticks(unq)
-       cb.set_ticklabels(unq)
-    plt.show()
-
 
 def cornerPlot(points, pos=1, s=0.2, title="", c="b", bg="k", lines=True):
     """
@@ -2311,15 +2256,15 @@ def cornerHistogram(
 
 
 class DirectedGraph(Assembly):
-    """A graph consists of a collection of vertices (without postional information)
-    and a collection of edges connecting pairs of vertices.
-    The task is to determine the vertex positions only based on their connections.
+    """A graph consists of a collection of nodes (without postional information)
+    and a collection of edges connecting pairs of nodes.
+    The task is to determine the node positions only based on their connections.
 
     This class is derived from class ``Assembly``, and it assembles 4 Mesh objects
-    representing the graph, the vertex labels, edge labels and edge arrows.
+    representing the graph, the node labels, edge labels and edge arrows.
 
     :param c: color of the Graph
-    :param int n: number of the initial set of vertices
+    :param int n: number of the initial set of nodes
     :param int,str layout: layout in ['2d', 'fast2d', 'clustering2d', 'circular',
                                       'circular3d', 'cone', 'force', 'tree']
 
@@ -2359,14 +2304,16 @@ class DirectedGraph(Assembly):
     Example:
 
         |lineage_graph| |lineage_graph.py|_
+
+        |graph_network| |graph_network.py|_
     """
     def __init__(self, **kargs):
         BaseActor.__init__(self)
 
-        self.vertices = []
+        self.nodes = []
         self.edges = []
 
-        self._vertexLabels = []  # holds strings
+        self._nodeLabels = []  # holds strings
         self._edgeLabels = []
         self.edgeOrientations = []
         self.edgeGlyphPosition = 0.6
@@ -2377,19 +2324,21 @@ class DirectedGraph(Assembly):
         self.rotY = 0
         self.rotZ = 0
 
-        self.arrowScale = 0.2
-        self.vertexLabelScale = None
+        self.arrowScale = 0.15
+        self.nodeLabelScale = None
         self.edgeLabelScale   = None
 
         self.mdg = vtk.vtkMutableDirectedGraph()
 
         n = kargs.pop('n', 0)
-        for i in range(n): self.addVertex()
+        for i in range(n): self.addNode()
 
         self._c = kargs.pop('c', (0.3,0.3,0.3))
 
         self.gl = vtk.vtkGraphLayout()
 
+        self.font = kargs.pop('font', '')
+
         s = kargs.pop('layout', '2d')
         if isinstance(s, int):
             ss = ['2d', 'fast2d', 'clustering2d', 'circular', 'circular3d',
@@ -2462,64 +2411,62 @@ class DirectedGraph(Assembly):
             self.rotX = 180
 
         else:
-            colors.printc("Cannot understand layout:", s, c=1)
-            colors.printc("Available layouts:", c=1)
-            colors.printc("[2d,fast2d,clustering2d,circular,circular3d,cone,force,tree]", c=1)
+            colors.printc("Cannot understand layout:", s, c='r')
+            colors.printc("Available layouts:", c='r')
+            colors.printc("[2d,fast2d,clustering2d,circular,circular3d,cone,force,tree]", c='r')
             raise RuntimeError()
 
         self.gl.SetLayoutStrategy(self.strategy)
 
         if len(kargs):
-            colors.printc("Cannot understand options:", kargs, c=1)
+            colors.printc("Cannot understand options:", kargs, c='r')
         return
 
 
-    def addVertex(self, label="id"):
-        """Add a new vertex to the Graph."""
-        v = self.mdg.AddVertex()
-        self.vertices.append(v)
+    def addNode(self, label="id"):
+        """Add a new node to the Graph."""
+        v = self.mdg.AddVertex() # vtk calls it vertex..
+        self.nodes.append(v)
         if label == 'id': label=int(v)
-        self._vertexLabels.append(str(label))
+        self._nodeLabels.append(str(label))
         return v
 
     def addEdge(self, v1, v2, label=""):
-        """Add a new edge between to vertices.
-        An extra vertex is created automatically if needed."""
-        nv = len(self.vertices)
+        """Add a new edge between to nodes.
+        An extra node is created automatically if needed."""
+        nv = len(self.nodes)
         if v1>=nv:
             for i in range(nv, v1+1):
-                self.addVertex()
-        nv = len(self.vertices)
+                self.addNode()
+        nv = len(self.nodes)
         if v2>=nv:
             for i in range(nv, v2+1):
-                self.addVertex()
+                self.addNode()
         e = self.mdg.AddEdge(v1,v2)
         self.edges.append(e)
         self._edgeLabels.append(str(label))
         return e
 
-    def addChild(self, v, vertexLabel="id", edgeLabel=""):
-        """Add a new edge to a new vertex.
-        The extra vertex is created automatically if needed."""
-        nv = len(self.vertices)
+    def addChild(self, v, nodeLabel="id", edgeLabel=""):
+        """Add a new edge to a new node as its child.
+        The extra node is created automatically if needed."""
+        nv = len(self.nodes)
         if v>=nv:
             for i in range(nv, v+1):
-                self.addVertex()
+                self.addNode()
         child = self.mdg.AddChild(v)
         self.edges.append((v,child))
-        self.vertices.append(child)
-        if vertexLabel == 'id': vertexLabel=int(child)
-        self._vertexLabels.append(str(vertexLabel))
+        self.nodes.append(child)
+        if nodeLabel == 'id': nodeLabel=int(child)
+        self._nodeLabels.append(str(nodeLabel))
         self._edgeLabels.append(str(edgeLabel))
         return child
 
-
     def build(self):
         """
         Build the DirectedGraph(Assembly).
         Accessory objects are also created for labels and arrows.
         """
-
         self.gl.SetZRange(self.zrange)
         self.gl.SetInputData(self.mdg)
         self.gl.Update()
@@ -2531,6 +2478,7 @@ class DirectedGraph(Assembly):
         graphToPolyData.Update()
 
         dgraph = Mesh(graphToPolyData.GetOutput(0))
+        dgraph.clean() # important to clean and have the same nr of points as vtxs
         dgraph.flat().color(self._c).lw(2)
         dgraph.name = "DirectedGraph"
 
@@ -2571,19 +2519,27 @@ class DirectedGraph(Assembly):
                 arrows.rotateZ(self.rotZ)
             arrows.name = "DirectedGraphArrows"
 
-        vertexLabels = dgraph.labels(self._vertexLabels,
-                                     scale=self.vertexLabelScale,
-                                     precision=0)
-        vertexLabels.color(self._c).pickable(True)
-        vertexLabels.name = "DirectedGraphVertexLabels"
+        nodeLabels = dgraph.labels(self._nodeLabels,
+                                    scale=self.nodeLabelScale,
+                                    precision=0,
+                                    font=self.font,
+                                    )
+        nodeLabels.color(self._c).pickable(True)
+        nodeLabels.name = "DirectedGraphNodeLabels"
 
         edgeLabels = dgraph.labels(self._edgeLabels,
-                                   cells=True,
-                                   scale=self.edgeLabelScale,
-                                   precision=0)
+                                    cells=True,
+                                    scale=self.edgeLabelScale,
+                                    precision=0,
+                                    font=self.font,
+                                    )
         edgeLabels.color(self._c).pickable(True)
         edgeLabels.name = "DirectedGraphEdgeLabels"
 
-        Assembly.__init__(self, [dgraph, vertexLabels, edgeLabels, arrows])
+        # # if edgeLabels.N() == 0:
+        # edgeLabels = None
+        # exit()
+
+        Assembly.__init__(self, [dgraph, nodeLabels, edgeLabels, arrows])
         self.name = "DirectedGraphAssembly"
-        return
+        return self
