@@ -9,6 +9,15 @@ General settings.
     ztitle = 'z'
     defaultAxesType = 4
 
+    # Set a default for the font to be used for axes, comments etc.
+    defaultFont = 'Normografo'
+    # Options are:
+    #    Biysk, Bongas, Calco, Comae, Kanopus, Glasgo, Inversionz, LionelOfParis,
+    #    LogoType, Normografo, Quikhand, SmartCouric, Theemim, VictorMono, VTK.
+    fontIsMono = True  # mono means that all letters occupy the same space slot horizontally
+    fontHSpacing = 1   # an horizontal stretching factor (affects both letters and words)
+    fontLSpacing = 0.1 # horizontal spacing inbetween letters (not words)
+
     # Scale magnification of the screenshot (must be an integer)
     screeshotScale = 1
     screenshotTransparentBackground = False
@@ -41,18 +50,6 @@ General settings.
     # Allow to interact with scene during interactor.Start() execution
     allowInteraction = True
 
-    # Flag-style label options
-    flagDelay    = 150       # popup delay in milliseconds
-    flagFont     = "Courier" # font type ("Arial", "Courier", "Times")
-    flagFontSize = 18
-    flagAngle    = 0
-    flagBold     = False
-    flagItalic   = True
-    flagShadow   = False
-    flagColor    = 'k'
-    flagJustification = 0
-    flagBackgroundColor = 'w'
-
     # Show a gray frame margin in multirendering windows
     showRendererFrame = True
     rendererFrameColor = None
@@ -63,18 +60,12 @@ General settings.
     # Use tex, matplotlib latex compiler
     usetex = False
 
-    # In 3D Text interpret _ and ^ as sub/super script
-    useModifiersInText = True
-
     # Qt embedding
     usingQt = False
 
-    # OpenVR rendering
+    # OpenVR rendering (untested)
     useOpenVR = False
 
-    # On some vtk versions/platforms points are rendered as ugly squares
-    renderPointsAsSpheres = True
-
     # Wrap lines in tubes
     renderLinesAsTubes = False
 
@@ -86,7 +77,7 @@ General settings.
     # Remove hidden lines when in wireframe mode
     hiddenLineRemoval = False
 
-    # For RectilinearGrid: show internal edges not only outline
+    # For Structured and RectilinearGrid: show internal edges not only outline
     visibleGridEdges = False
 
     # Turn on/off the automatic repositioning of lights as the camera moves.
@@ -99,6 +90,9 @@ General settings.
     maxNumberOfPeels= 8     # maximum number of rendering passes
     occlusionRatio  = 0.0   # occlusion ratio, 0 = exact image.
 
+    # Turn on/off nvidia FXAA anti-aliasing, if supported.
+    useFXAA = None          # either True or False. None sets the system default
+
     # Use a polygon/edges offset to possibly resolve conflicts in rendering
     usePolygonOffset    = False
     polygonOffsetFactor = 0.1
@@ -107,15 +101,19 @@ General settings.
     # Interpolate scalars to render them smoothly
     interpolateScalarsBeforeMapping = True
 
-    # Turn on/off nvidia FXAA anti-aliasing, if supported.
-    useFXAA = None          # either True or False. None sets the system default
-
     # Set parallel projection On or Off (place camera to infinity, no perspective effects)
     useParallelProjection = False
 
-    # Path to Voro++ library, http://math.lbl.gov/voro++
-    voro_path = '/usr/local/bin'
-
+    # AnnotatedCube axis type nr. 5 options:
+    annotatedCubeColor      = (0.75, 0.75, 0.75)
+    annotatedCubeTextColor  = None # use default, otherwise specify a single color
+    annotatedCubeTextScale  = 0.2
+    annotatedCubeXPlusText  = "right"
+    annotatedCubeXMinusText = "left "
+    annotatedCubeYPlusText  = "front"
+    annotatedCubeYMinusText = "back "
+    annotatedCubeZPlusText  = " top "
+    annotatedCubeZMinusText = "bttom"
 
 Usage example:
 
@@ -140,6 +138,12 @@ ytitle = 'y'
 ztitle = 'z'
 defaultAxesType = None
 
+# Set a default for the font to be used for axes, comments etc.
+defaultFont = 'Normografo'
+fontIsMono = True
+fontHSpacing = 1
+fontLSpacing = 0.1
+
 # Scale magnification of the screenshot (must be an integer)
 screeshotScale = 1
 screenshotTransparentBackground = False
@@ -157,37 +161,21 @@ interactorStyle = None
 # Allow to interact with scene during interactor.Start() execution
 allowInteraction = True
 
-# Flag-style label options
-flagDelay = 150         # popup delay in milliseconds
-flagFont = "Courier"   # font type ("Arial", "Courier", "Times")
-flagFontSize = 18
-flagJustification = 0
-flagAngle = 0
-flagBold = False
-flagItalic = True
-flagShadow = False
-flagColor = 'k'
-flagBackgroundColor = 'w'
-
 # Show a gray frame margin in multirendering windows
 showRendererFrame = True
 rendererFrameColor = None
+rendererFrameAlpha = 0.5
+rendererFrameWidth = 0.5
 
 # Use tex, matplotlib latex compiler
 usetex = False
 
-# In 3D Text interpret _ and ^ as sub/super script
-useModifiersInText = True
-
 # Qt embedding
 usingQt = False
 
 # OpenVR rendering
 useOpenVR = False
 
-# On some vtk versions/platforms points are rendered as ugly squares
-renderPointsAsSpheres = True
-
 # Wrap lines in tubes
 renderLinesAsTubes = False
 
@@ -199,7 +187,7 @@ pointSmoothing = False
 lineSmoothing = False
 polygonSmoothing = False
 
-# For (Un)Structured and RectilinearGrid: show internal edges not only outline
+# For Structured and RectilinearGrid: show internal edges not only outline
 visibleGridEdges = False
 
 # Turn on/off the automatic repositioning of lights as the camera moves.
@@ -213,6 +201,9 @@ multiSamples    = 0
 maxNumberOfPeels= 8
 occlusionRatio  = 0.0
 
+# Turn on/off nvidia FXAA anti-aliasing, if supported.
+useFXAA = None           # either True or False. None sets the system default
+
 # Use a polygon/edges offset to possibly resolve conflicts in rendering
 usePolygonOffset = False
 polygonOffsetFactor = 0.1
@@ -221,39 +212,59 @@ polygonOffsetUnits  = 0.1
 # Interpolate scalars to render them smoothly
 interpolateScalarsBeforeMapping = True
 
-# Turn on/off nvidia FXAA anti-aliasing, if supported.
-useFXAA = None           # either True or False. None sets the system default
-
 # Set parallel projection On or Off (place camera to infinity, no perspective effects)
 useParallelProjection = False
 
 # In multirendering mode set the position of the horizontal of vertical splitting [0,1]
 windowSplittingPosition = None
 
-# Path to Voro++ library, http://math.lbl.gov/voro++
-voro_path = '/usr/local/bin'
-
+# AnnotatedCube axis type 5 customization:
+annotatedCubeColor      = (0.75, 0.75, 0.75)
+annotatedCubeTextColor  = None # use default, otherwise specify a single color
+annotatedCubeTextScale  = 0.2
+annotatedCubeXPlusText  = "right"
+annotatedCubeXMinusText = "left "
+annotatedCubeYPlusText  = "front"
+annotatedCubeYMinusText = "back "
+annotatedCubeZPlusText  = " top "
+annotatedCubeZMinusText = "bttom"
 
 ####################################################################################
 # notebook support with K3D
 notebookBackend = None
 notebook_plotter = None
 
-plotter_instance = None
-collectable_actors = []
-
-
 ####################################################################################
-_cdir = os.path.dirname(__file__)
+flagDelay = 150 # values will be superseded
+flagFont = "Courier"
+flagFontSize = 18
+flagJustification = 0
+flagAngle = 0
+flagBold = False
+flagItalic = True
+flagShadow = False
+flagColor = 'k'
+flagBackgroundColor = 'w'
+legendSize = 0.2
+legendBC = (0.96, 0.96, 0.9)
+legendPos = 2
+legendFont = ""
 
-textures_path = os.path.join(_cdir, "textures/")
+installdir = os.path.dirname(__file__)
+
+textures_path = os.path.join(installdir, "textures/")
 textures = []
 
-fonts_path = os.path.join(_cdir, "fonts/")
+fonts_path = os.path.join(installdir, "fonts/")
 fonts = []
 
-datadir = os.path.join(_cdir, "examples/data/")
+#datadir = "/home/musy/Dropbox/Public/vktwork/vedo_data/"
+datadir = "https://vedo.embl.es/examples/data/"
 
+plotter_instances = []
+
+plotter_instance = None
+collectable_actors = []
 
 ####################################################################################
 def embedWindow(backend='k3d', verbose=True):
@@ -335,6 +346,7 @@ def _init():
         textures.append(tfn)
 
     for f in os.listdir(fonts_path):
+        if '.npz' in f: continue
         fonts.append(f.split(".")[0])
     fonts = list(sorted(fonts))
 
