@@ -33,11 +33,11 @@ class UGrid(vtk.vtkActor, BaseGrid):
         elif isinstance(inputobj, str):
             from vedo.io import download, loadUnStructuredGrid
             if "https://" in inputobj:
-                inputobj = download(inputobj)
+                inputobj = download(inputobj, verbose=False)
             self._data = loadUnStructuredGrid(inputobj)
 
         else:
-            colors.printc("UGrid(): cannot understand input type:\n", inputtype, c=1)
+            colors.printc("UGrid(): cannot understand input type:\n", inputtype, c='r')
             return
 
         self._mapper = vtk.vtkPolyDataMapper()
