@@ -1,5 +1,5 @@
 from __future__ import division, print_function
-import glob
+import glob, os
 import numpy as np
 import vtk
 import vedo.colors as colors
@@ -15,7 +15,208 @@ Submodule extending the ``vtkVolume`` object functionality.
     + docs._defs
 )
 
-__all__ = ["Volume"]
+__all__ = ["Volume",
+           "mesh2Volume",
+           "volumeFromMesh",
+           "interpolateToVolume",
+           "signedDistanceFromPointCloud",
+          ]
+
+
+
+def mesh2Volume(mesh, spacing=(1, 1, 1)):
+    """
+    Convert a mesh it into a ``Volume``
+    where the foreground (exterior) voxels value is 1 and the background
+    (interior) voxels value is 0.
+    Internally the ``vtkPolyDataToImageStencil`` class is used.
+
+    |mesh2volume| |mesh2volume.py|_
+    """
+    # https://vtk.org/Wiki/VTK/Examples/Cxx/PolyData/PolyDataToImageData
+    pd = mesh.polydata()
+
+    whiteImage = vtk.vtkImageData()
+    bounds = pd.GetBounds()
+
+    whiteImage.SetSpacing(spacing)
+
+    # compute dimensions
+    dim = [0, 0, 0]
+    for i in [0, 1, 2]:
+        dim[i] = int(np.ceil((bounds[i * 2 + 1] - bounds[i * 2]) / spacing[i]))
+    whiteImage.SetDimensions(dim)
+    whiteImage.SetExtent(0, dim[0] - 1, 0, dim[1] - 1, 0, dim[2] - 1)
+
+    origin = [0, 0, 0]
+    origin[0] = bounds[0] + spacing[0] / 2
+    origin[1] = bounds[2] + spacing[1] / 2
+    origin[2] = bounds[4] + spacing[2] / 2
+    whiteImage.SetOrigin(origin)
+    whiteImage.AllocateScalars(vtk.VTK_UNSIGNED_CHAR, 1)
+
+    # fill the image with foreground voxels:
+    inval = 255
+    count = whiteImage.GetNumberOfPoints()
+    for i in range(count):
+        whiteImage.GetPointData().GetScalars().SetTuple1(i, inval)
+
+    # polygonal data --> image stencil:
+    pol2stenc = vtk.vtkPolyDataToImageStencil()
+    pol2stenc.SetInputData(pd)
+    pol2stenc.SetOutputOrigin(origin)
+    pol2stenc.SetOutputSpacing(spacing)
+    pol2stenc.SetOutputWholeExtent(whiteImage.GetExtent())
+    pol2stenc.Update()
+
+    # cut the corresponding white image and set the background:
+    outval = 0
+    imgstenc = vtk.vtkImageStencil()
+    imgstenc.SetInputData(whiteImage)
+    imgstenc.SetStencilConnection(pol2stenc.GetOutputPort())
+    imgstenc.ReverseStencilOff()
+    imgstenc.SetBackgroundValue(outval)
+    imgstenc.Update()
+    return Volume(imgstenc.GetOutput())
+
+
+def volumeFromMesh(mesh, bounds=None, dims=(20,20,20), signed=True, negate=False):
+    """
+    Compute signed distances over a volume from an input mesh.
+    The output is a ``Volume`` object whose voxels contains the signed distance from
+    the mesh.
+
+    :param list bounds: bounds of the output volume.
+    :param list dims: dimensions (nr. of voxels) of the output volume.
+
+    See example script: |volumeFromMesh.py|_
+    """
+    if bounds is None:
+        bounds = mesh.GetBounds()
+    sx = (bounds[1]-bounds[0])/dims[0]
+    sy = (bounds[3]-bounds[2])/dims[1]
+    sz = (bounds[5]-bounds[4])/dims[2]
+
+    img = vtk.vtkImageData()
+    img.SetDimensions(dims)
+    img.SetSpacing(sx, sy, sz)
+    img.SetOrigin(bounds[0], bounds[2], bounds[4])
+    img.AllocateScalars(vtk.VTK_FLOAT, 1)
+
+    imp = vtk.vtkImplicitPolyDataDistance()
+    imp.SetInput(mesh.polydata())
+    b4 = bounds[4]
+    r2 = range(dims[2])
+
+    for i in range(dims[0]):
+        x = i*sx+bounds[0]
+        for j in range(dims[1]):
+            y = j*sy+bounds[2]
+            for k in r2:
+                v = imp.EvaluateFunction((x, y, k*sz+b4))
+                if signed:
+                    if negate:
+                        v = -v
+                else:
+                    v = abs(v)
+                img.SetScalarComponentFromFloat(i,j,k, 0, v)
+
+    vol = Volume(img)
+    vol.name = "VolumeFromMesh"
+    return vol
+
+def interpolateToVolume(mesh, kernel='shepard', radius=None,
+                       bounds=None, nullValue=None,
+                       dims=(20,20,20)):
+    """
+    Generate a ``Volume`` by interpolating a scalar
+    or vector field which is only known on a scattered set of points or mesh.
+    Available interpolation kernels are: shepard, gaussian, voronoi, linear.
+
+    :param str kernel: interpolation kernel type [shepard]
+    :param float radius: radius of the local search
+    :param list bounds: bounding box of the output Volume object
+    :param list dims: dimensions of the output Volume object
+    :param float nullValue: value to be assigned to invalid points
+
+    |interpolateVolume| |interpolateVolume.py|_
+    """
+    if isinstance(mesh, vtk.vtkPolyData):
+        output = mesh
+    else:
+        output = mesh.polydata()
+
+    # Create a probe volume
+    probe = vtk.vtkImageData()
+    probe.SetDimensions(dims)
+    if bounds is None:
+        bounds = output.GetBounds()
+    probe.SetOrigin(bounds[0],bounds[2],bounds[4])
+    probe.SetSpacing((bounds[1]-bounds[0])/dims[0],
+                     (bounds[3]-bounds[2])/dims[1],
+                     (bounds[5]-bounds[4])/dims[2])
+
+    if radius is None:
+        radius = min(bounds[1]-bounds[0], bounds[3]-bounds[2], bounds[5]-bounds[4])/3
+
+    locator = vtk.vtkPointLocator()
+    locator.SetDataSet(output)
+    locator.BuildLocator()
+
+    if kernel == 'shepard':
+        kern = vtk.vtkShepardKernel()
+        kern.SetPowerParameter(2)
+        kern.SetRadius(radius)
+    elif kernel == 'gaussian':
+        kern = vtk.vtkGaussianKernel()
+        kern.SetRadius(radius)
+    elif kernel == 'voronoi':
+        kern = vtk.vtkVoronoiKernel()
+    elif kernel == 'linear':
+        kern = vtk.vtkLinearKernel()
+        kern.SetRadius(radius)
+    else:
+        print('Error in interpolateToVolume, available kernels are:')
+        print(' [shepard, gaussian, voronoi, linear]')
+        raise RuntimeError()
+
+    interpolator = vtk.vtkPointInterpolator()
+    interpolator.SetInputData(probe)
+    interpolator.SetSourceData(output)
+    interpolator.SetKernel(kern)
+    interpolator.SetLocator(locator)
+    if nullValue is not None:
+        interpolator.SetNullValue(nullValue)
+    else:
+        interpolator.SetNullPointsStrategyToClosestPoint()
+    interpolator.Update()
+    return Volume(interpolator.GetOutput())
+
+
+
+def signedDistanceFromPointCloud(mesh, maxradius=None, bounds=None, dims=(20,20,20)):
+    """
+    Compute signed distances over a volume from an input point cloud.
+    The output is a ``Volume`` object whose voxels contains the signed distance from
+    the cloud.
+
+    :param float maxradius: how far out to propagate distance calculation
+    :param list bounds: volume bounds.
+    :param list dims: dimensions (nr. of voxels) of the output volume.
+    """
+    if bounds is None:
+        bounds = mesh.GetBounds()
+    if maxradius is None:
+        maxradius = mesh.diagonalSize()/10.
+    dist = vtk.vtkSignedDistance()
+    dist.SetInputData(mesh.polydata(True))
+    dist.SetRadius(maxradius)
+    dist.SetBounds(bounds)
+    dist.SetDimensions(dims)
+    dist.Update()
+    vol = Volume(dist.GetOutput())
+    vol.name = "signedDistanceVolume"
+    return vol
 
 
 ##########################################################################
@@ -68,7 +269,9 @@ class Volume(vtk.vtkVolume, BaseGrid):
 
             if "https://" in inputobj:
                 from vedo.io import download
-                inputobj = download(inputobj) # fpath
+                inputobj = download(inputobj, verbose=False) # fpath
+            elif os.path.isfile(inputobj):
+                pass
             else:
                 inputobj = sorted(glob.glob(inputobj))
 
@@ -116,7 +319,7 @@ class Volume(vtk.vtkVolume, BaseGrid):
                     img.SetDimensions(dims)
                 else:
                     if len(inputobj.shape)==1:
-                        colors.printc("Error: must set dimensions (dims keyword) in Volume.", c=1)
+                        colors.printc("Error: must set dimensions (dims keyword) in Volume.", c='r')
                         raise RuntimeError()
                     img.SetDimensions(inputobj.shape)
                 img.GetPointData().SetScalars(varr)
@@ -142,11 +345,13 @@ class Volume(vtk.vtkVolume, BaseGrid):
             img = inputobj.GetOutput()
 
         elif isinstance(inputobj, str):
-            from vedo.io import loadImageData
+            from vedo.io import loadImageData, download
+            if "https://" in inputobj:
+                inputobj = download(inputobj, verbose=False)
             img = loadImageData(inputobj)
 
         else:
-            colors.printc("Volume(): cannot understand input type:\n", inputtype, c=1)
+            colors.printc("Volume(): cannot understand input type:\n", inputtype, c='r')
             return
 
         ###################
@@ -279,8 +484,10 @@ class Volume(vtk.vtkVolume, BaseGrid):
         else:
             return np.array(self._data.GetSpacing())
 
-    def origin(self, s=None):                                ### superseedes base.origin()
-        """Set/get the origin of the volumetric dataset.""" ### DIFFERENT from base.origin()!
+    def origin(self, s=None):
+        """Set/get the origin of the volumetric dataset."""
+        ### superseedes base.origin()
+        ### DIFFERENT from base.origin()!
         if s is not None:
             self._data.SetOrigin(s)
             return self
@@ -330,85 +537,6 @@ class Volume(vtk.vtkVolume, BaseGrid):
         return self._update(rsp.GetOutput())
 
 
-    # def color(self, col):
-    #     """Assign a color or a set of colors to a volume along the range of the scalar value.
-    #     A single constant color can also be assigned.
-    #     Any matplotlib color map name is also accepted, e.g. ``volume.color('jet')``.
-
-    #     E.g.: say that your voxel scalar runs from -3 to 6,
-    #     and you want -3 to show red and 1.5 violet and 6 green, then just set:
-
-    #     ``volume.color(['red', 'violet', 'green'])``
-    #     """
-    #     smin, smax = self._data.GetScalarRange()
-    #     ctf = self.GetProperty().GetRGBTransferFunction()
-    #     ctf.RemoveAllPoints()
-    #     self._color = col
-
-    #     if utils.isSequence(col):
-    #         for i, ci in enumerate(col):
-    #             r, g, b = colors.getColor(ci)
-    #             x = smin + (smax - smin) * i / (len(col) - 1)
-    #             ctf.AddRGBPoint(x, r, g, b)
-    #             #colors.printc('\tcolor at', round(x, 1),
-    #             #              '\tset to', colors.getColorName((r, g, b)), c='w', bold=0)
-    #     elif isinstance(col, str):
-    #         if col in colors.colors.keys() or col in colors.color_nicks.keys():
-    #             r, g, b = colors.getColor(col)
-    #             ctf.AddRGBPoint(smin, r,g,b) # constant color
-    #             ctf.AddRGBPoint(smax, r,g,b)
-    #         elif colors._mapscales:
-    #             for x in np.linspace(smin, smax, num=64, endpoint=True):
-    #                 r,g,b = colors.colorMap(x, name=col, vmin=smin, vmax=smax)
-    #                 ctf.AddRGBPoint(x, r, g, b)
-    #     elif isinstance(col, int):
-    #         r, g, b = colors.getColor(col)
-    #         ctf.AddRGBPoint(smin, r,g,b) # constant color
-    #         ctf.AddRGBPoint(smax, r,g,b)
-    #     else:
-    #         colors.printc("volume.color(): unknown input type:", col, c=1)
-    #     return self
-
-    # def alpha(self, alpha):
-    #     """Assign a set of tranparencies to a volume along the range of the scalar value.
-    #     A single constant value can also be assigned.
-
-    #     E.g.: say alpha=(0.0, 0.3, 0.9, 1) and the scalar range goes from -10 to 150.
-    #     Then all voxels with a value close to -10 will be completely transparent, voxels at 1/4
-    #     of the range will get an alpha equal to 0.3 and voxels with value close to 150
-    #     will be completely opaque.
-
-    #     As a second option one can set explicit (x, alpha_x) pairs to define the transfer function.
-    #     E.g.: say alpha=[(-5, 0), (35, 0.4) (123,0.9)] and the scalar range goes from -10 to 150.
-    #     Then all voxels below -5 will be completely transparent, voxels with a scalar value of 35
-    #     will get an opacity of 40% and above 123 alpha is set to 90%.
-    #     """
-    #     smin, smax = self._data.GetScalarRange()
-    #     otf = self.GetProperty().GetScalarOpacity()
-    #     otf.RemoveAllPoints()
-    #     self._alpha = alpha
-
-    #     if utils.isSequence(alpha):
-    #         alpha = np.array(alpha)
-    #         if len(alpha.shape)==1: # user passing a flat list e.g. (0.0, 0.3, 0.9, 1)
-    #             for i, al in enumerate(alpha):
-    #                 xalpha = smin + (smax - smin) * i / (len(alpha) - 1)
-    #                 # Create transfer mapping scalar value to opacity
-    #                 otf.AddPoint(xalpha, al)
-    #         elif len(alpha.shape)==2: # user passing [(x0,alpha0), ...]
-    #             otf.AddPoint(smin, alpha[0][1])
-    #             for xalpha, al in alpha:
-    #                 # Create transfer mapping scalar value to opacity
-    #                 otf.AddPoint(xalpha, al)
-    #             otf.AddPoint(smax, alpha[-1][1])
-    #         #colors.printc("alpha at", round(xalpha, 1), "\tset to", al)
-
-    #     else:
-    #         otf.AddPoint(smin, alpha) # constant alpha
-    #         otf.AddPoint(smax, alpha)
-
-    #     return self
-
     def alphaGradient(self, alphaGrad):
         """
         Assign a set of tranparencies to a volume's gradient
@@ -453,21 +581,6 @@ class Volume(vtk.vtkVolume, BaseGrid):
             gotf.AddPoint(smax, alphaGrad)
         return self
 
-    # def alphaUnit(self, u=None):
-    #     """Defines light attenuation per unit length. Default is 1.
-    #     The larger the unit length, the further light has to travel to attenuate the same amount.
-
-    #     E.g., if you set the unit distance to 0, you will get full opacity.
-    #     It means that when light travels 0 distance it's already attenuated a finite amount.
-    #     Thus, any finite distance should attenuate all light.
-    #     The larger you make the unit distance, the more transparent the rendering becomes.
-    #     """
-    #     if u is None:
-    #         return self.GetProperty().GetScalarOpacityUnitDistance()
-    #     else:
-    #         self.GetProperty().SetScalarOpacityUnitDistance(u)
-    #         return self
-
     def interpolation(self, itype):
         """
         Set interpolation type.
@@ -517,17 +630,19 @@ class Volume(vtk.vtkVolume, BaseGrid):
         return self._update(th.GetOutput())
 
     def crop(self,
-             top=None, bottom=None,
-             right=None, left=None,
-             front=None, back=None, VOI=()):
+             left=None, right=None,
+             back=None, front=None,
+             bottom=None, top=None,
+             VOI=()
+            ):
         """Crop a ``Volume`` object.
 
-        :param float top:    fraction to crop from the top plane (positive z)
-        :param float bottom: fraction to crop from the bottom plane (negative z)
-        :param float front:  fraction to crop from the front plane (positive y)
-        :param float back:   fraction to crop from the back plane (negative y)
-        :param float right:  fraction to crop from the right plane (positive x)
         :param float left:   fraction to crop from the left plane (negative x)
+        :param float right:  fraction to crop from the right plane (positive x)
+        :param float back:   fraction to crop from the back plane (negative y)
+        :param float front:  fraction to crop from the front plane (positive y)
+        :param float bottom: fraction to crop from the bottom plane (negative z)
+        :param float top:    fraction to crop from the top plane (positive z)
         :param list VOI:     extract Volume Of Interest expressed in voxel numbers
 
             Eg.: vol.crop(VOI=(xmin, xmax, ymin, ymax, zmin, zmax)) # all integers nrs
@@ -639,7 +754,7 @@ class Volume(vtk.vtkVolume, BaseGrid):
         elif axis.lower() == "z":
             ff.SetFilteredAxis(2)
         else:
-            colors.printc("~times Error in mirror(): mirror must be set to x, y, z or n.", c=1)
+            colors.printc("\times Error in mirror(): mirror must be set to x, y, z or n.", c='r')
             raise RuntimeError()
         ff.Update()
         return self._update(ff.GetOutput())
@@ -819,7 +934,7 @@ class Volume(vtk.vtkVolume, BaseGrid):
         elif op in ["atan2"]:
             mat.SetOperationToATAN2()
         else:
-            colors.printc("~times Error in volumeOperation: unknown operation", operation, c=1)
+            colors.printc("\times Error in volumeOperation: unknown operation", operation, c='r')
             raise RuntimeError()
         mat.Update()
         return self._update(mat.GetOutput())
@@ -829,8 +944,8 @@ class Volume(vtk.vtkVolume, BaseGrid):
         """
         Low-pass and high-pass filtering become trivial in the frequency domain.
         A portion of the pixels/voxels are simply masked or attenuated.
-        This function applies a high pass Butterworth filter that attenuates the frequency domain
-        image with the function
+        This function applies a high pass Butterworth filter that attenuates the
+        frequency domain image with the function
 
         |G_Of_Omega|
 
@@ -855,7 +970,7 @@ class Volume(vtk.vtkVolume, BaseGrid):
         if highcutoff:
             butterworthLowPass = vtk.vtkImageButterworthLowPass()
             butterworthLowPass.SetInputData(out)
-            butterworthLowPass.SetCutOff(highcutoff) # actually inverted..(?)
+            butterworthLowPass.SetCutOff(highcutoff)
             butterworthLowPass.SetOrder(order)
             butterworthLowPass.Update()
             out = butterworthLowPass.GetOutput()
@@ -1003,4 +1118,3 @@ class Volume(vtk.vtkVolume, BaseGrid):
 
 
 
-
